-- gel.e: my motley collection of graphics elements
-- j.babor@gns.cri.nz
-- 98-03-13

include graphics.e

global function maxlx(sequence s)
    -- return max row length for (possibly) non-rectangular 2-d image (=width)
    integer l
    l=length(s[1])
    for i=2 to length(s) do
        if length(s[i])>l then l=length(s[i]) end if end for
    return l end function

global procedure merge_image(sequence p,sequence im,integer c)
    -- display 2-d sequence of pixels im at point p, similar to display_image,
    -- except image pixels of color c are considered transparent
    sequence bg
    integer lx
    for i=1 to length(im) do
        lx=length(im[i])
        bg=get_pixel(p&lx)
        for j=1 to lx do
            if im[i][j]=c then im[i][j]=bg[j] end if end for
        pixel(im[i],p)
        p[2]=p[2]+1 end for end procedure

global procedure rect(sequence p1,sequence p2,integer c)
    -- draw hollow rectangle of color c, defined by diagonal points p1, p2
    draw_line(c,{p1,{p2[1],p1[2]},p2,{p1[1],p2[2]},p1}) end procedure

global procedure box(sequence p1,sequence p2,object c)
    -- draw filled rectangle defined by diagonal points p1, p2
    -- c is either an integer, indicating the fill color,
    -- or a 2-dimensional texture color sequence
    sequence s
    integer dx,lx,ly,mx,my,n
    if sequence(c) then -- textured box
        dx=p2[1]-p1[1]+1
        ly=length(c)
        lx=length(c[1])
        n=floor(dx/lx)
        s=c
        for i=1 to ly do
            for j=1 to n do
                s[i]=s[i]&c[i] end for end for
        mx=1+remainder(p1[1],lx)
        for i=1 to ly do
            s[i]=c[i][mx..lx] & s[i] end for
        for y=p1[2] to p2[2] do
            my=1+remainder(y,ly)
            pixel(s[my][1..dx],{p1[1],y}) end for
    else polygon(c,1,{p1,{p2[1],p1[2]},p2,{p1[1],p2[2]}}) end if
end procedure   -- box

global procedure frame(sequence p1, sequence p2,sequence ec)
    -- draw frame: hollow rectangle defined by diagonal points p1,p2
    -- ec is sequence of edge colors sequences: {{lt1,lt2,..},{rb1,rb2,..}}
    -- diagonally from left top to right bottom
    integer m,n
    m=length(ec[1])
    n=length(ec[2])
    if n then
        for i=0 to n-1 do draw_line(ec[2][n-i],{{p1[1]+i,p2[2]-i},p2-i,
            {p2[1]-i,p1[2]+i}}) end for end if
    if m then
        for i=0 to m-1 do draw_line(ec[1][i+1],{{p1[1]+i,p2[2]-i},p1+i,
            {p2[1]-i,p1[2]+i}}) end for end if end procedure -- frame

global procedure tile(sequence p1, sequence p2,object c,sequence ec)
    -- draw edged filled rectangle defined by diagonal points p1,p2
    -- c is either face color or texture sequence
    -- ec is sequence of edge colors sequences: {{lt1,lt2,..},{rb1,rb2,..}}
    -- diagonally from left top to right bottom
    integer m,n
    m=length(ec[1])
    n=length(ec[2])
    if n then
        for i=0 to n-1 do draw_line(ec[2][n-i],{{p1[1]+i,p2[2]-i},p2-i,
            {p2[1]-i,p1[2]+i}}) end for end if
    if m then
        for i=0 to m-1 do draw_line(ec[1][i+1],{{p1[1]+i,p2[2]-i},p1+i,
            {p2[1]-i,p1[2]+i}}) end for end if
    box(p1+m,p2-n,c) end procedure -- tile


-- xor graphics ----------------------------------------------------------------

global procedure xor_pixel(object c, sequence xy)
    --  pixel(s) xor-ed with background, same syntax as pixel()
    if sequence(c) then pixel(xor_bits(get_pixel(xy&length(c)),c),xy)
    else pixel(xor_bits(get_pixel(xy),c),xy) end if end procedure

global procedure xor_hline(integer c,sequence p)
    --  horizontal line xor-ed with background
    --  c is color
    --  p={x1,y1,x2}
    sequence s,xy
    integer len
    len=p[3]-p[1]+1
    xy=p[1..2]
    s=repeat(c,len)
    pixel(xor_bits(get_pixel(xy&len),s),xy) end procedure

global procedure xor_vline(integer c,sequence p)
    --  vertical line xor-ed with background
    --  c is color
    --  p={x1,y1,y2}
    integer x
    x=p[1]
    for y=p[2] to p[3] do
        pixel(xor_bits(get_pixel({x,y}),c),{x,y}) end for end procedure

global procedure xor_rect(sequence p1,sequence p2,integer c)
    -- rectangular hollow box xor-ed with background
    -- c is color, p1 & p2 are top_left & bottom_right diagonal points
    sequence s
    integer len,x1,x2,y1,y2
    x1=p1[1]
    x2=p2[1]
    y1=p1[2]
    y2=p2[2]
    if x1>x2 then   -- safety first
        len=x2
        x2=x1
        x1=len end if
    if y1>y2 then
        len=y2
        y2=y1
        y1=len end if
    len=x2-x1+1
    s=repeat(c,len)
    pixel(xor_bits(get_pixel({x1,y1,len}),s),{x1,y1})
    pixel(xor_bits(get_pixel({x1,y2,len}),s),{x1,y2})
    for y=y1+1 to y2-1 do
        pixel(xor_bits(get_pixel({x1,y}),c),{x1,y})
        pixel(xor_bits(get_pixel({x2,y}),c),{x2,y}) end for end procedure

global procedure xor_image(sequence p,sequence im)
    --  same format as display_image(): 2-d sequence of pixels im
    --  is xor-ed with screen (background) at point p
    for i=1 to length(im) do
        pixel(xor_bits(get_pixel(p&length(im[i])),im[i]),p)
        p[2]=p[2]+1 end for end procedure
