-- pbutton.e: pbutton (pictorial button) widget
-- j.babor@gns.cri.nz
-- v 0.1    98-03-16

include pic.e
include panel.e
include faces.e

global function pbutton(sequence p1, sequence p2, sequence g1, integer f1,
    sequence g2, integer f2, integer f)
    -- create new pbutton, defined by diagonal points p1,p2
    -- g1, g2 are alternative 2-d images: raised & sunken
    -- flags f1, f2 indicate mode of display for respective images:
    --     put, merged or xor-ed
    -- flag f indicates initial pbutton state:   0=popped,
    --                                          1=pushed
    -- face & edge colors & font setting will be current defaults
    -- return pbutton sequence
    sequence p,pl,pc1,pc2
    integer lx,ly
    lx=maxlx(g1)
    ly=length(g1)
    p=floor((p1+p2-{lx-1,ly-1})/2)
    pc1=pic(p,g1,f1)
    lx=maxlx(g2)
    ly=length(g2)
    p=floor((p1+p2-{lx-1,ly-1})/2)+1
    pc2=pic(p,g2,f2)
    pl=panel(p1,p2,colors[1],f)
    return {{pl,pc1,pc2},f,p1,p2} end function

global procedure show_pbutton(sequence w)
    show_panel(w[WS][1])
    if w[MF] then show_pic(w[WS][3])    -- 'sunken'
    else show_pic(w[WS][2]) end if      -- 'popped'
end procedure   -- show_pbutton

global procedure click_pbutton(sequence w)
    -- only popped pbuttons can be clicked
    -- clicked pbutton remains pushed only while rat button is pressed
    sequence s
    if w[MF]=0 then                         -- popped
        s=w                                 -- save original state
        s[WS][1][MF]=w[WS][1][MF]+1         -- switch to pushed look
        s[MF]=1
        show_pbutton(s)
        wait_while(LEFT_BUTTON_PRESSED)
        show_pbutton(w) end if end procedure-- switch back to original look

global function toggle_pbutton(sequence w)
    -- pbuttons are similar to check boxes
    -- any pbutton can be toggled, irrespective of its original state
    -- return changed pbutton
    integer f,g
    f=w[MF]
    g=w[WS][1][MF]
    if f then
        f=f-1
        g=g-1
    else
        f=f+1
        g=g+1 end if
    w[MF]=f
    w[WS][1][MF]=g
    show_pbutton(w)
    wait_while(LEFT_BUTTON_PRESSED)
    return w end function

global function frame_pbutton(sequence w)
    sequence p
    integer f
    f=w[WS][1][MF]
    if f<2 then
        f=f+2
        w[WS][1][MF]=f
        p=w[WS][1]      -- panel
        hide_rat()
        frame(p[LT],p[RB],p[EC][f+1])
        show_rat() end if
    return w end function

global function deframe_pbutton(sequence w)
    sequence p      -- panel
    integer f
    f=w[WS][1][MF]
    if f>=2 then
        f=f-2
        w[WS][1][MF]=f
        p=w[WS][1]
        hide_rat()
        frame(p[LT],p[RB],p[EC][f+1])
        show_rat() end if
    return w end function
