-- rat.e : mouse related routines
-- j.babor@gns.cri.nz
-- v 1.00   98-03-08

--------------------------------------------------------------------------------
-- general mouse routines: usable with most mouse graphics systems
--------------------------------------------------------------------------------

include mouse.e
include machine.e

global constant
    LEFT_BUTTON_PRESSED=1,
    RIGHT_BUTTON_PRESSED=2,
    MIDDLE_BUTTON_PRESSED=4

global procedure set_rat_xy(integer x, integer y)
    -- places the rat pointer where you want it
    sequence rl -- list of register values
    rl = repeat(0,10)
    rl[REG_AX] = 4
    rl[REG_CX] = x
    rl[REG_DX] = y
    rl = dos_interrupt(#33,rl)
end procedure  -- set_rat_xy

global procedure confine_rat(sequence p1, sequence p2)
-- confine rat pointer to rectangle given by diagonal points p1,p2
    sequence rl -- list of register values
    rl = repeat(0,10)
    rl[REG_AX] = 7
    rl[REG_CX] = p1[1]
    rl[REG_DX] = p2[1]
    rl = dos_interrupt(#33,rl)
    rl[REG_AX] = 8
    rl[REG_CX] = p1[2]
    rl[REG_DX] = p2[2]
    rl = dos_interrupt(#33,rl)
end procedure  -- confine_rat

global function poll_rat()
-- returns rat state: {button_state,x_coord,y_coord}
-- button state:    0 .. no button pressed
--                  1 .. left button pressed
--                  2 .. right button pressed
--                  4 .. middle button pressed (if your rat has it)
    sequence  rl -- list of register values
    rl = repeat(0,10)
    rl[REG_AX] = 3
    rl = dos_interrupt(#33,rl)
    return {rl[REG_BX], rl[REG_CX], rl[REG_DX]}
end function   -- poll_rat

global function get_rat_speed()
--  from Colin Taylor's mousfunc.e
--  returns {x, y, a} where:
--      x = current horizontal speed setting (range 0 - 256)
--      y = current vertical speed setting (range 0 - 256)
--      a = current acceleration threshold setting (range 0 - 256)
    sequence rl
    rl = repeat(0,10)
    rl[REG_AX] = 27
    rl = dos_interrupt(#33,rl)
    return {rl[REG_BX], rl[REG_CX], rl[REG_DX]}
end function  -- get_rat_speed

global procedure set_rat_speed(sequence speed)
-- from Colin Taylor's mousfunc.e
-- sets rat speed parameters to speed {x, y, a} where:
--      x = new horizontal speed setting (range 0 - 256)
--      y = new vertical speed setting (range 0 - 256)
--      a = new acceleration threshold setting (range 0 - 256)
    sequence rl
    rl = repeat(0,10)
    rl[REG_AX] = 26
    rl[REG_BX] = speed[1]  -- horiz. speed
    rl[REG_CX] = speed[2]  -- vert. speed
    rl[REG_DX] = speed[3]  -- accel. threshhold
    rl = dos_interrupt(#33,rl)
end procedure  -- set_rat_speed


--------------------------------------------------------------------------------
-- rat specific routines: *simple* superVGA xor pointer & drag box system
--------------------------------------------------------------------------------

global constant
    ARROW={{1,1},
        {8},
        {8,8},
        {8,7,8},
        {8,7,7,8},
        {8,7,7,7,8},
        {8,7,7,7,7,8},
        {8,7,7,7,7,7,8},
        {8,7,7,7,8,8,8,8},
        {8,7,8,8,8},
        {8,8,0,8,8},
        {8,0,0,0,8,8},
        {0,0,0,0,8,8},
        {0,0,0,0,0,8,8},
        {0,0,0,0,0,8,8}},

    ARROW1={{1,1},
        {8},
        {8,8},
        {8,7,8},
        {8,7,7,8},
        {8,7,7,7,8},
        {8,7,7,7,7,8},
        {8,7,7,7,7,7,8},
        {8,7,7,7,8,8,8,8},
        {8,7,8,8,8},
        {8,8,0,8,8,8},
        {8,0,0,8,8,8,8},
        {0,0,0,8}},

    TRIANGLE={{1,1},
        {8,8,8,8,8,8,8,8,8,8,8},
        {8,7,7,7,7,7,7,7,7,7},
        {8,7,7,7,7,7,7,7,7},
        {8,7,7,7,7,7,7,7},
        {8,7,7,7,7,7,7},
        {8,7,7,7,7,7},
        {8,7,7,7,7},
        {8,7,7,7},
        {8,7,7},
        {8,7},
        {8}},

    CROSSHAIR={{6,6},
        {0,0,0,0,0,8},
        {0,0,0,0,0,8},
        {0,0,0,0,0,8},
        {0,0,0,0,0,8},
        {},
        {8,8,8,8,0,0,0,8,8,8,8},
        {},
        {0,0,0,0,0,8},
        {0,0,0,0,0,8},
        {0,0,0,0,0,8},
        {0,0,0,0,0,8}},

    IBEAM={{2,6},
        {8,8,8},
        {0,8},
        {0,8},
        {0,8},
        {0,8},
        {0,8},
        {0,8},
        {0,8},
        {0,8},
        {0,8},
        {8,8,8}},

    LTRB={{6,6},
        {8,8,8,8,8},
        {8,7,7,8},
        {8,7,7,7,8},
        {8,8,7,7,7,8},
        {8,0,8,7,7,7,8},
        {0,0,0,8,7,7,7,8},
        {0,0,0,0,8,7,7,7,8,0,8},
        {0,0,0,0,0,8,7,7,7,8,8},
        {0,0,0,0,0,0,8,7,7,7,8},
        {0,0,0,0,0,0,0,8,7,7,8},
        {0,0,0,0,0,0,8,8,8,8,8}},

    RTLB={{6,6},
        {0,0,0,0,0,0,8,8,8,8,8},
        {0,0,0,0,0,0,0,8,7,7,8},
        {0,0,0,0,0,0,8,7,7,7,8},
        {0,0,0,0,0,8,7,7,7,8,8},
        {0,0,0,0,8,7,7,7,8,0,8},
        {0,0,0,8,7,7,7,8},
        {8,0,8,7,7,7,8},
        {8,8,7,7,7,8},
        {8,7,7,7,8},
        {8,7,7,8},
        {8,8,8,8,8}},

    LR={{8,4},
        {0,0,0,8,0,0,0,0,0,0,0,8},
        {0,0,8,8,0,0,0,0,0,0,0,8,8},
        {0,8,7,8,8,8,8,8,8,8,8,8,7,8},
        {8,7,7,7,7,7,7,7,7,7,7,7,7,7,8},
        {0,8,7,8,8,8,8,8,8,8,8,8,7,8},
        {0,0,8,8,0,0,0,0,0,0,0,8,8},
        {0,0,0,8,0,0,0,0,0,0,0,8}},

    TB={{4,8},
        {0,0,0,8},
        {0,0,8,7,8},
        {0,8,7,7,7,8},
        {8,8,8,7,8,8,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {8,8,8,7,8,8,8},
        {0,8,7,7,7,8},
        {0,0,8,7,8},
        {0,0,0,8}},

    PENCIL={{1,1},
        {12,12,6,6,6},
        {12,6,6,6,8,6},
        {6,6,6,8,6,8,6},
        {6,6,8,8,8,6,8,6},
        {6,8,6,8,8,8,6,8,6},
        {0,6,8,6,8,8,8,6,8,6},
        {0,0,6,8,6,8,8,8,6,8,6},
        {0,0,0,6,8,6,8,8,8,6,8,6},
        {0,0,0,0,6,8,6,8,8,8,6,8,6},
        {0,0,0,0,0,6,8,6,8,8,8,6},
        {0,0,0,0,0,0,6,8,6,8,6},
        {0,0,0,0,0,0,0,6,8,6},
        {0,0,0,0,0,0,0,0,6}},


    HGLASS={{5,7},
        {8,8,8,8,8,8,8,8,8},
        {8,0,0,0,0,0,0,0,8},
        {8,0,0,0,0,0,0,0,8},
        {0,8,7,7,7,7,7,8},
        {0,0,8,7,7,7,8},
        {0,0,0,8,7,8},
        {0,0,0,0,8},
        {0,0,0,8,0,8},
        {0,0,8,0,0,0,8},
        {0,8,0,0,7,0,0,8},
        {8,0,7,7,7,7,7,0,8},
        {8,7,7,7,7,7,7,7,8},
        {8,8,8,8,8,8,8,8,8}},

    HAND={{3,1},
        {0,0,8,8,8},
        {0,0,8,0,8},
        {0,0,8,0,8},
        {0,0,8,0,8},
        {0,0,8,0,8},
        {0,0,8,0,8},
        {0,0,8,0,8,8,8,8,8,8,8},
        {0,0,8,0,8,0,8,0,8,0,8},
        {8,8,8,0,8,0,8,0,8,0,8},
        {8,0,8,0,8,0,8,0,8,0,8},
        {8,0,8,0,0,0,0,0,0,0,8},
        {8,0,8,0,0,0,0,0,0,0,8},
        {8,0,0,0,0,0,0,0,0,0,8},
        {8,0,0,0,0,0,0,0,0,0,8},
        {8,0,0,0,0,0,0,0,0,0,8},
        {8,8,8,8,8,8,8,8,8,8,8}},

    HAND1={{3,1},
        {0,0,8,8,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8},
        {0,0,8,7,8,8,8,8,8,8,8},
        {0,0,8,7,8,7,8,7,8,7,8},
        {8,8,8,7,8,7,8,7,8,7,8},
        {8,7,8,7,8,7,8,7,8,7,8},
        {8,7,8,7,7,7,7,7,7,7,8},
        {8,7,8,7,7,7,7,7,7,7,8},
        {8,7,7,7,7,7,7,7,7,7,8},
        {8,7,7,7,7,7,7,7,7,7,8},
        {8,7,7,7,7,7,7,7,7,7,8},
        {8,8,8,8,8,8,8,8,8,8,8}},

    HAND2={{3,1},
        {0,0,8,8,8},
        {0,0,8,0,8},
        {0,0,8,0,8},
        {0,0,8,0,8},
        {0,0,8,0,8},
        {0,0,8,0,8,8,8},
        {0,0,8,0,8,0,8,8,8},
        {0,0,8,0,8,0,8,0,8,8,8},
        {8,8,8,0,8,0,8,0,8,0,8},
        {8,0,8,0,0,0,0,0,8,0,8},
        {8,0,8,0,0,0,0,0,0,0,8},
        {8,0,8,0,0,0,0,0,0,0,8},
        {8,0,0,0,0,0,0,0,0,0,8},
        {8,0,0,0,0,0,0,0,0,0,8},
        {8,0,0,0,0,0,0,0,0,0,8},
        {8,8,8,8,8,8,8,8,8,8,8}}

global sequence
    rat_state   -- rat state={button_state,x,y}

sequence
    db,         -- drag box state: {x1,y1,x2,y2,color}
    srs,        -- saved rat state
    hotspot,    -- hot spot coordinates within pointer image
    pointer     -- 2-d pixel color pointer image

procedure draw_rat()
    sequence p
    p=srs[2..3]-hotspot
    for i=1 to length(pointer) do
        pixel(xor_bits(get_pixel(p&length(pointer[i])),pointer[i]),p)
        p[2]=p[2]+1 end for
end procedure

global procedure show_rat()
-- display pointer if it is currently hidden
-- if pointer is already displayed redraw it only if it has changed location
    rat_state=poll_rat()     -- get rat state
    if not srs[1] or rat_state[2]!=srs[2] or rat_state[3]!=srs[3] then
        if srs[1] then draw_rat() end if
        srs=1 & rat_state[2..3]
        draw_rat() end if
end procedure -- show_rat

global procedure hide_rat()
    if srs[1] then draw_rat() end if
    srs[1]=0
end procedure -- hide_rat

global procedure rat_pointer(sequence p)
    -- select new pointer defined by p
    integer flag
    flag=srs[1]         -- save rat pointer state, shown or hidden
    hide_rat()
    hotspot=p[1]-1
    pointer=p[2..length(p)]
    if flag then show_rat() end if   -- restore original state
end procedure -- rat_pointer

global procedure wait_while (integer pressed_button)
    -- wait while specified rat button remains pressed
    show_rat()  -- also updates rat state variable rat_state using poll_rat()
    while and_bits(rat_state[1],pressed_button) do
        show_rat() end while    -- keep rat pointer alive
end procedure -- wait_while

global function left_button_pressed()
-- return true (=1) if left button pressed
    return and_bits(rat_state[1],LEFT_BUTTON_PRESSED)
end function

global function right_button_pressed()
-- return true (=1) if right button pressed
    return and_bits(rat_state[1],RIGHT_BUTTON_PRESSED)!=0
end function

global function track_zones(sequence zones)
-- scan zones for rat pointer presence
-- zones is sequence of rectangular zones to be tracked
-- each zone is defined by its top_left & bottom_right diagonal points
-- return index of the *first* zone that includes rat pointer, or zero
-- if none found
    show_rat()  -- also updates rat state variable rat_state using poll_rat()
    for i=1 to length(zones) do
        if rat_state[2]>=zones[i][1][1] and
           rat_state[2]<=zones[i][2][1] and
           rat_state[3]>=zones[i][1][2] and
           rat_state[3]<=zones[i][2][2] then return i end if end for
    return 0
end function -- track_zones


-- drag box routines -----------------------------------------------------------

procedure safe_xor_rect(sequence p1,sequence p2,integer c)
    -- safe, because order of p1,p2 is immaterial
    -- rectangular hollow (line) box xor-ed with background
    -- c is color, p1 & p2 are top_left & bottom_right diagonal points
    sequence s
    integer len,x1,x2,y1,y2
    x1=p1[1]
    x2=p2[1]
    y1=p1[2]
    y2=p2[2]
    if x1>x2 then   -- safety first
        len=x2
        x2=x1
        x1=len end if
    if y1>y2 then
        len=y2
        y2=y1
        y1=len end if
    len=x2-x1+1
    s=repeat(c,len)
    pixel(xor_bits(get_pixel({x1,y1,len}),s),{x1,y1})
    pixel(xor_bits(get_pixel({x1,y2,len}),s),{x1,y2})
    for y=y1+1 to y2-1 do
        pixel(xor_bits(get_pixel({x1,y}),c),{x1,y})
        pixel(xor_bits(get_pixel({x2,y}),c),{x2,y}) end for end procedure

global procedure erase_drag_box()
-- erase drag box if one is drawn
    integer rf      -- rat flag
    if db[5] then
        rf=srs[1]
        if rf then draw_rat() end if    -- hide rat if shown
        safe_xor_rect({db[1],db[2]},{db[3],db[4]},db[5])
        db[5]=0
        if rf then draw_rat() end if end if -- restore rat if previously shown
end procedure -- erase_drag_box

global function drag_box(integer c,integer button_pressed)
-- integrates drag box with rat xor pointer: continually updates drag box
-- while specified rat button remains pressed
-- c is color of drag box (true only on BLACK background)
-- see button pressed constants at top of this file
-- return drag box data (={color,x1,y1,x2,y2})
    erase_drag_box()
    show_rat()  -- also updates rat state variable rat_state using poll_rat()
    if and_bits(button_pressed,rat_state[1]) then
        db={rat_state[2],rat_state[3],rat_state[2],rat_state[3],c}
        safe_xor_rect({db[1],db[2]},{db[3],db[4]},c)
        while and_bits(rat_state[1],button_pressed) do
            if rat_state[2]!=db[3] or rat_state[3]!=db[4] then
                hide_rat()
                safe_xor_rect({db[1],db[2]},{db[3],db[4]},c)
                db[3]=rat_state[2]
                db[4]=rat_state[3]
                safe_xor_rect({db[1],db[2]},{db[3],db[4]},c) end if
            show_rat() end while end if
    return db
end function -- drag_box


-- init rat & drag boxes -------------------------------------------------------

srs={0,0,0}     -- init superVGA rat: {hidden=0/shown=1,x,y}
rat_pointer(ARROW)
--rat_pointer(HAND1)
db={0,0,0,0,0}  -- last drawn drag box: {x1,y1,x2,y2,color}; none if color=0
