-- widgets: mickey mouse user interface
-- j.babor@gns.cri.nz
-- v 0.1   98-03-18

include gel.e       -- graphics elements
include image.e
include rat.e
include font.e

global sequence
    colors,         -- alternative face colors
    edges,          -- alternative edge colors
    fcolors,        -- alternative text colors
    fscolors,       -- alternative shadow colors
    fmargins,       -- left & right text margins
    fparams         -- font parameters

global integer
    maxx,maxy,      -- screen limits in selected mode
    fhandle         -- font handle

global constant
-- defaults corresponding to global variables of same name ---------------------
    COLORS={8,7,9,1,12,4,15,{{8,15},{15,8}}},
    EDGES={{{11,8},{7,3}},          -- popped look (left top to right bot)
           {{3,7},{8,11}},          -- pushed
           {{0,11},{7,0}},          -- framed popped
           {{0,7},{11,0}}},         -- framed pushed
    FHANDLE=fload("sysa13.f"),      -- default font
    FCOLORS={0,1,4,14,15},          -- standard text colors
    FSCOLORS={7,3,1,0,4,0,11,7},    -- shadow colors matching COLORS above
    FPARAMS={1,1,1,0,0,1,1,0,0,0},  -- see font.e for deatails
    FMARGINS={0,1024},              -- default left & right text margins
-- default mix for primary face color (color 8) --------------------------------
    RGB={48,48,44},
-- general fields --------------------------------------------------------------
    WS=1,           -- list of included (secondary) widgets
    MF=2,           -- general purpose message field (flags etc)
-- draw fields -----------------------------------------------------------------
    LT=3,           -- left top corner coordinates
    RB=4,           -- right bottom corner coordinates
    CO=5,           -- face color
    EC=6,           -- edge colors
    IM=7,           -- foreground or saved background image
-- picture fields --------------------------------------------------------------
--  LT=3,           -- left top corner coordinates of displayed picture
    PC=4,           -- picture: 2-d pixel sequence
-- font (text) fields ----------------------------------------------------------
    FX=3,           -- fx, text pointer
    FY=4,           -- fy, text pointer
    FT=5,           -- text or label
    FH=6,           -- handle
    FA=7,           -- attribute
    FC=8,           -- text color
    FS=9,           -- shadow color
    FB=10,          -- background (paper) color
    FP=11,          -- parameters
    FJ=12,          -- justification
    FM=13,          -- margins
    FW=14           -- line wrap flag

global procedure set_rgb(sequence rgb)
    -- set color mix for standard face colors
    -- rgb={red,green,blue} is palette for primary face color (color 8)
    object junk
    junk=palette(8,rgb)
    junk=palette(3,rgb*0.33)
    junk=palette(7,rgb*0.67)
    junk=palette(11,(rgb+64)/2) end procedure

global procedure init_widgets(integer mode)
    -- initialize widgets: set graphics mode
    --                     reset and expand gray scale (takes over cyans)
    --                     confine rat to within selected screen limits
    object vc
    if graphics_mode(mode) then
        puts(1,"Requested graphics mode not available!\n") end if
    -- expanded gray scale - using two cyan slots (never liked cyan...)
    set_rgb(RGB)
    vc=video_config()
    maxx=vc[VC_XPIXELS]-1
    maxy=vc[VC_YPIXELS]-1
    if mode=19 then confine_rat({8,0},{maxx,maxy}) -- compensate for eu bug
    else confine_rat({0,0},{maxx,maxy}) end if end procedure

global function track_widgets(sequence s)
    -- s is sequence of id's of 'trackable' widgets
    -- trackable widgets have 'left_top' & 'right_bottom' (LT,RB) fields
    -- return index of *first* widget with rat pointer in its zone, or
    -- zero if none
    show_rat()  -- also updates rat state variable rat_state using poll_mouse()
    for i=1 to length(s) do
        if rat_state[2]>=s[i][LT][1] and
           rat_state[2]<=s[i][RB][1] and
           rat_state[3]>=s[i][LT][2] and
           rat_state[3]<=s[i][RB][2] then return i end if end for
    return 0 end function


-- set defaults ----------------------------------------------------------------

colors=COLORS       -- default face colors
edges=EDGES         -- default edge colors
fhandle=FHANDLE     -- default font
fcolors=FCOLORS     -- default text colors
fscolors=FSCOLORS   -- default shadow colors
fparams=FPARAMS     -- default font parameters
fmargins=FMARGINS   -- default line margins
